@@ -24,6 +24,7 @@ BUILD_IMAGE="false"
 
 VM_MEMORY_SIZE="3072"
 VM_DOMAIN_TYPE=qemu
+export VM_DISK_CACHE="unsafe"
 TEST_VM_NUM_NODES=1
 USE_CIRROS=true
 TESTING_USER=cirros
@@ -35,6 +36,47 @@ INVENTORY_DHCP_STATIC_IP=false
 DOWNLOAD_IPA=true
 CREATE_IPA_IMAGE=false
 WRITE_INTERFACES_FILE=true
+PROVISION_WAIT_TIMEOUT=${PROVISION_WAIT_TIMEOUT:-900}
+NOAUTH_MODE=true
+ENABLE_KEYSTONE=false
+CLOUD_CONFIG=""
+WAIT_FOR_DEPLOY=true
+ZUUL_BRANCH=${ZUUL_BRANCH:-}
+
+# This sets up the MySQL database like it's done for all OpenStack
+# projects for CI testing.
+mysql_setup() {
+    # The root password for the MySQL database; pass it in via
+    # MYSQL_ROOT_PW.
+    local DB_ROOT_PW=${MYSQL_ROOT_PW:-insecure_slave}
+
+    # This user and its password are used by the tests, if you change it,
+    # your tests might fail.
+    local DB_USER=openstack_citest
+    local DB_PW=openstack_citest
+
+    sudo -H mysqladmin -u root password $DB_ROOT_PW
+
+    # It's best practice to remove anonymous users from the database.  If
+    # an anonymous user exists, then it matches first for connections and
+    # other connections from that host will not work.
+    sudo -H mysql -u root -p$DB_ROOT_PW -h localhost -e "
+        DELETE FROM mysql.user WHERE User='';
+        FLUSH PRIVILEGES;
+        GRANT ALL PRIVILEGES ON *.*
+            TO '$DB_USER'@'%' identified by '$DB_PW' WITH GRANT OPTION;"
+
+    # Now create our database.
+    mysql -u $DB_USER -p$DB_PW -h 127.0.0.1 -e "
+        SET default_storage_engine=MYISAM;
+        DROP DATABASE IF EXISTS openstack_citest;
+        CREATE DATABASE openstack_citest CHARACTER SET utf8;"
+}
+
+# Setup openstack_citest database if run in OpenStack CI.
+if [ "$ZUUL_BRANCH" != "" ] ; then
+    mysql_setup
+fi
 
 # NOTE(cinerama): We could remove this if we change the CI job to use
 # USE_DHCP, BUILD_IMAGE, etc.
@@ -45,28 +87,27 @@ elif [ $SOURCE = "test-bifrost-venv.sh" ]; then
      USE_VENV="true"
 elif [ $SOURCE = "test-bifrost-build-images.sh" ]; then
      BUILD_IMAGE="true"
+elif [ $SOURCE = "test-bifrost-keystone-auth.sh" ]; then
+     ENABLE_KEYSTONE="true"
 fi
 
-# Source Ansible
-# NOTE(TheJulia): Ansible stable-1.9 source method tosses an error deep
-# under the hood which -x will detect, so for this step, we need to suspend
-# and then re-enable the feature.
-set +x +o nounset
 if [ ${USE_VENV} = "true" ]; then
     export VENV=/opt/stack/bifrost
     export PATH=${VENV}/bin:${PATH}
     $SCRIPT_HOME/env-setup.sh
+    # Note(cinerama): activate is not compatible with "set -u";
+    # disable it just for this line.
+    set +u
     source /opt/stack/bifrost/bin/activate
+    set -u
     ANSIBLE=${VENV}/bin/ansible-playbook
     ENABLE_VENV="true"
 else
     $SCRIPT_HOME/env-setup.sh
-    source ${ANSIBLE_INSTALL_ROOT}/ansible/hacking/env-setup
-    ANSIBLE=$(which ansible-playbook)
+    ANSIBLE=${HOME}/.local/bin/ansible-playbook
 fi
-set -x -o nounset
 
-# Adjust options for DHCP or create VM tests
+# Adjust options for DHCP, VM, or Keystone tests
 if [ ${USE_DHCP} = "true" ]; then
     VM_MEMORY_SIZE="1024"
     ENABLE_INSPECTOR=false
@@ -83,6 +124,9 @@ elif [ ${BUILD_IMAGE} = "true" ]; then
     INSPECT_NODES=false
     DOWNLOAD_IPA=false
     CREATE_IPA_IMAGE=true
+elif [ ${ENABLE_KEYSTONE} = "true" ]; then
+    NOAUTH_MODE=false
+    CLOUD_CONFIG="-e cloud_name=bifrost"
 fi
 
 # Change working directory
@@ -135,7 +179,12 @@ ${ANSIBLE} -vvvv \
     -e inspect_nodes=${INSPECT_NODES} \
     -e download_ipa=${DOWNLOAD_IPA} \
     -e create_ipa_image=${CREATE_IPA_IMAGE} \
-    -e write_interfaces_file=${WRITE_INTERFACES_FILE}
+    -e write_interfaces_file=${WRITE_INTERFACES_FILE} \
+    -e wait_timeout=${PROVISION_WAIT_TIMEOUT} \
+    -e noauth_mode=${NOAUTH_MODE} \
+    -e enable_keystone=${ENABLE_KEYSTONE} \
+    -e wait_for_node_deploy=${WAIT_FOR_DEPLOY} \
+    ${CLOUD_CONFIG}
 EXITCODE=$?
 
 if [ $EXITCODE != 0 ]; then
